@@ -8,13 +8,6 @@
  */
 #undef	HAVE_STRERROR
 
-/*
- * NLIST_HAS_N_NAME is defined iff a struct nlist has an n_name member.
- * If it doesn't then we assume it has an n_un member which, in turn,
- * has an n_name member.
- */
-#undef	NLIST_HAS_N_NAME
-
 /*
  * HAVE_SYS_SELECT_H is defined iff we have the include file sys/select.h.
  */
@@ -43,7 +36,6 @@
 /*
  * We define this (as 'unsigned int') if the OS doesn't.
  */
-
 #undef socklen_t
 
 @BOTTOM@
@@ -53,3 +45,4 @@
 #include "fake/sys-select.h"
 #include "fake/environ.h"
 #include "fake/sysexits.h"
+#include "fake/langinfo.h"
