@@ -1,4 +1,4 @@
-/* Copyright (c) 1993-2005 by Richard Kelsey and Jonathan Rees.
+/* Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees.
    See file COPYING. */
 
 #include <stdlib.h>
@@ -460,9 +460,10 @@ s48_system(s48_value string)
 {
   return s48_enter_integer(system((string == S48_FALSE)
 				  ? NULL
-				  : s48_extract_string(string)));
+				  : s48_extract_byte_vector(string)));
 }
 
+
 /********************************/
 /*
  * Raising exceptions.  We push the arguments on the stack end then throw out
@@ -505,27 +506,12 @@ s48_raise_argument_type_error(s48_value value) {
   s48_raise_scheme_exception(S48_EXCEPTION_WRONG_TYPE_ARGUMENT, 1, value);
 }
 
-/* Superceded name for preceding function, retained for compatibility. */
-
-void
-s48_raise_argtype_error(s48_value value) {
-  s48_raise_scheme_exception(S48_EXCEPTION_WRONG_TYPE_ARGUMENT, 1, value);
-}
-
 void
 s48_raise_argument_number_error(s48_value value, s48_value min, s48_value max) {
   s48_raise_scheme_exception(S48_EXCEPTION_WRONG_NUMBER_OF_ARGUMENTS,
 			     3, value, min, max);
 }
 
-/* Superceded name for preceding function, retained for compatibility. */
-
-void
-s48_raise_argnumber_error(s48_value value, s48_value min, s48_value max) {
-  s48_raise_scheme_exception(S48_EXCEPTION_WRONG_NUMBER_OF_ARGUMENTS,
-			     3, value, min, max);
-}
-
 void
 s48_raise_range_error(s48_value value, s48_value min, s48_value max) {
   s48_raise_scheme_exception(S48_EXCEPTION_INDEX_OUT_OF_RANGE,
@@ -541,13 +527,13 @@ void
 s48_raise_os_error(int the_errno) {
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 2,
 			     s48_enter_fixnum(the_errno),
-			     s48_enter_string(strerror(the_errno)));
+			     s48_enter_string_latin_1(strerror(the_errno)));
 }
 
 void
 s48_raise_string_os_error(char *reason) {
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 1,
-			     s48_enter_string(reason));
+			     s48_enter_string_latin_1(reason));
 }
 
 void
@@ -722,6 +708,27 @@ s48_extract_integer(s48_value value)
   else s48_raise_argument_type_error(value);
 }
 
+unsigned long
+s48_extract_unsigned_integer(s48_value value)
+{
+  if (S48_FIXNUM_P(value))
+    {
+      long fixnum = S48_UNSAFE_EXTRACT_FIXNUM(value);
+      if (fixnum < 0)
+	s48_raise_argument_type_error(value);
+      return (unsigned long) fixnum;
+    }
+
+  if (S48_BIGNUM_P(value)){
+    bignum_type bignum = S48_ADDRESS_AFTER_HEADER(value, long);
+    
+    if (! s48_bignum_fits_in_word_p(bignum, 32, 0))
+      s48_raise_argument_type_error(value);
+    else return s48_bignum_to_ulong(bignum);
+  }
+  else s48_raise_argument_type_error(value);
+}
+
 /*
  * Doubles and characters are straightforward.
  */
@@ -747,17 +754,20 @@ s48_extract_double(s48_value s48_double)
 }
 
 s48_value
-s48_enter_char(unsigned char a_char)
+s48_enter_char(long a_char)
 {
-  if (a_char > 255)
+  if (! ((a_char >= 0)
+	 && ((a_char <= 0xd7ff)
+	     || ((a_char >= 0xe000) && (a_char <= 0x10ffff)))))
+    /* #### not quite */
     s48_raise_range_error(s48_enter_fixnum(a_char),
 			  s48_enter_fixnum(0),
-			  s48_enter_fixnum(255));
+			  s48_enter_fixnum(0x10ffff));
 
   return S48_UNSAFE_ENTER_CHAR(a_char);
 }
 
-unsigned char
+long
 s48_extract_char(s48_value a_char)
 {
   if (! S48_CHAR_P(a_char))
@@ -765,7 +775,7 @@ s48_extract_char(s48_value a_char)
   
   return S48_UNSAFE_EXTRACT_CHAR(a_char);
 }
-  
+
 /********************************/
 /* Allocation */
 
@@ -780,6 +790,13 @@ s48_enter_pointer(void *pointer)
   return obj;
 }
 
+void*
+s48_extract_pointer(s48_value sch_pointer)
+{
+  S48_CHECK_VALUE(sch_pointer);
+  return *(S48_ADDRESS_AFTER_HEADER(sch_pointer, void *));
+}
+
 s48_value
 s48_cons(s48_value v1, s48_value v2)
 {
@@ -811,33 +828,6 @@ s48_make_weak_pointer(s48_value value)
   return obj;
 }
 
-/*
- * Entering and extracting strings.
- */
-
-s48_value
-s48_enter_substring(char *str, long length)
-{
-  s48_value obj = s48_allocate_stob(S48_STOBTYPE_STRING, length + 1);
-  memcpy(S48_UNSAFE_EXTRACT_STRING(obj), str, length);
-  *(S48_UNSAFE_EXTRACT_STRING(obj) + length) = '\0';
-  return obj;
-}
-
-s48_value
-s48_enter_string(char *str)
-{
-  return s48_enter_substring(str, strlen(str));
-}
-
-char *
-s48_extract_string(s48_value string)
-{
-  S48_CHECK_STRING(string);
-
-  return S48_UNSAFE_EXTRACT_STRING(string);
-}
-
 /*
  * Entering and extracting byte vectors.
  */
@@ -846,7 +836,7 @@ s48_value
 s48_enter_byte_vector(char *bytes, long length)
 {
   s48_value obj = s48_allocate_stob(S48_STOBTYPE_BYTE_VECTOR, length);
-  memcpy(S48_UNSAFE_EXTRACT_STRING(obj), bytes, length);
+  memcpy(S48_UNSAFE_EXTRACT_BYTE_VECTOR(obj), bytes, length);
   return obj;
 }
 
@@ -855,7 +845,7 @@ s48_extract_byte_vector(s48_value byte_vector)
 {
   S48_CHECK_VALUE(byte_vector);
 
-  return S48_UNSAFE_EXTRACT_STRING(byte_vector);
+  return S48_UNSAFE_EXTRACT_BYTE_VECTOR(byte_vector);
 }
 
 /*
@@ -863,11 +853,13 @@ s48_extract_byte_vector(s48_value byte_vector)
  */
 
 s48_value
-s48_make_string(int length, char init)
+s48_make_string(int length, long init)
 {
-  s48_value obj = s48_allocate_stob(S48_STOBTYPE_STRING, length+1);
-  memset(S48_UNSAFE_EXTRACT_STRING(obj), init, length);
-  S48_UNSAFE_EXTRACT_STRING(obj)[length] = '\0';
+  int i;
+  s48_value obj = s48_allocate_string(length);
+  /* We should probably offer a VM function for this. */
+  for (i = 0; i < length; ++i)
+    s48_string_set(obj, i, init);
   return obj;
 }
 
@@ -895,6 +887,21 @@ s48_make_byte_vector(int length)
     return s48_allocate_stob(S48_STOBTYPE_BYTE_VECTOR, length);
 }
 
+s48_value
+s48_enter_byte_substring(char *str, long length)
+{
+  s48_value obj = s48_allocate_stob(S48_STOBTYPE_BYTE_VECTOR, length + 1);
+  memcpy(S48_UNSAFE_EXTRACT_BYTE_VECTOR(obj), str, length);
+  *(S48_UNSAFE_EXTRACT_BYTE_VECTOR(obj) + length) = '\0';
+  return obj;
+}
+
+s48_value
+s48_enter_byte_string(char *str)
+{
+  return s48_enter_byte_substring(str, strlen(str));
+}
+
 s48_value
 s48_make_record(s48_value type_shared_binding)
 {
