@@ -1,4 +1,4 @@
-/* Copyright (c) 1993-2005 by Richard Kelsey and Jonathan Rees.
+/* Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees.
    See file COPYING. */
 
 /*
@@ -21,7 +21,8 @@
 #include "posix.h"
 #include "unix.h"
 
-extern void		s48_init_posix_proc(void);
+extern void		s48_init_posix_proc(void),
+			s48_uninit_posix_proc(void);
 static s48_value	posix_fork(void),
 			posix_exec(s48_value program, s48_value lookup_p,
 				   s48_value env, s48_value args),
@@ -30,14 +31,16 @@ static s48_value	posix_fork(void),
 			posix_integer_to_signal(s48_value sig_int),
 			posix_initialize_named_signals(void),
 			posix_request_interrupts(s48_value int_number),  
-			posix_cancel_interrupt_request(s48_value int_number),  
+			posix_cancel_interrupt_request(s48_value sch_signal),
   			posix_kill(s48_value sch_pid, s48_value sch_signal);
 
 static s48_value	enter_signal(int signal);
 static int		extract_signal(s48_value sch_signal);
-static void		signal_map_init();
+static void		signal_map_init(void);
+static void		signal_map_uninit(void);
+static void		cancel_interrupt_requests(void);
 
-static char		**enter_string_array(s48_value strings),
+static char		**enter_byte_vector_array(s48_value strings),
 			*add_dot_slash(char *name);
 
 /*
@@ -118,6 +121,14 @@ s48_init_posix_proc(void)
   signal_map_init();
 }
 
+void
+s48_uninit_posix_proc(void)
+{
+  /* this will lose our signal handlers without reinstalling them; too bad */
+  cancel_interrupt_requests();
+  signal_map_uninit();
+}
+
 /*
  * Box a process id in a Scheme record.
  */
@@ -311,11 +322,11 @@ static s48_value
 posix_exec(s48_value program, s48_value lookup_p,
 	   s48_value env, s48_value args)
 {
-  char **c_args = enter_string_array(args);
+  char **c_args = enter_byte_vector_array(args);
   char *c_program, *real_c_program;
   int status;
 
-  c_program = s48_extract_string(program);
+  c_program = s48_extract_byte_vector(program);
 
   s48_stop_alarm_interrupts();
 
@@ -326,7 +337,7 @@ posix_exec(s48_value program, s48_value lookup_p,
       status = execvp(c_program, c_args);
     }
   else {
-    char **c_env = enter_string_array(env);
+    char **c_env = enter_byte_vector_array(env);
     
     if (NULL == strchr(c_program, '/'))
       real_c_program = add_dot_slash(c_program);
@@ -350,25 +361,25 @@ posix_exec(s48_value program, s48_value lookup_p,
 }
 
 /*
- * Convert a list of strings into an array of char pointers.
+ * Convert a list of byte vectors into an array of char pointers.
  */
 
 static char **
-enter_string_array(s48_value strings)
+enter_byte_vector_array(s48_value vectors)
 {
-  int length = S48_UNSAFE_EXTRACT_FIXNUM(s48_length(strings));
+  int length = S48_UNSAFE_EXTRACT_FIXNUM(s48_length(vectors));
   char **result = (char **)malloc((length + 1) * sizeof(char *));
   int i;
 
   if (result == NULL)
     s48_raise_out_of_memory_error();
   
-  for(i = 0; i < length; i++, strings = S48_UNSAFE_CDR(strings)) {
-    s48_value string = S48_UNSAFE_CAR(strings);
-    if (! S48_STRING_P(string)) {
+  for(i = 0; i < length; i++, vectors = S48_UNSAFE_CDR(vectors)) {
+    s48_value vector = S48_UNSAFE_CAR(vectors);
+    if (! S48_BYTE_VECTOR_P(vector)) {
       free(result);
-      s48_raise_argument_type_error(string); }
-    result[i] = S48_UNSAFE_EXTRACT_STRING(string); }
+      s48_raise_argument_type_error(vector); }
+    result[i] = S48_UNSAFE_EXTRACT_BYTE_VECTOR(vector); }
   result[length] = NULL;
 
   return result;
@@ -398,15 +409,6 @@ add_dot_slash(char *name)
  * Signals
  */
 
-/*
- * These need to be replaced with something that really blocks interrupts.
- * I don't know what that should be.  This is needed in c/unix/events.c as
- * well.
- */
-
-#define block_interrupts()
-#define allow_interrupts()
-
 /*
  * Simple front for kill().  We have to retry if interrupted.
  */
@@ -457,6 +459,12 @@ signal_map_init()
 #include "s48_signals.h"
 }
 
+static void
+signal_map_uninit(void)
+{
+  free(signal_map);
+}
+
 /*
  * Converts from an OS signal to a canonical signal number.
  * We return -1 if there is no matching named signal.
@@ -614,17 +622,19 @@ extract_signal(s48_value sch_signal)
 static void
 generic_interrupt_catcher(int signum)
 {
+  extern void s48_add_os_signal(long);
   s48_add_os_signal(signum);
 
   switch (signum) {
   case SIGINT: {
-    extern void		s48_when_keyboard_interrupt(int ign);
     s48_when_keyboard_interrupt(0);
     break; }
   case SIGALRM: {
-    extern void		s48_when_alarm_interrupt(int ign);
     s48_when_alarm_interrupt(0);
     break; }
+  case SIG_EXTERNAL_EVENT: {
+    s48_when_external_event_interrupt(0);
+    break; }
   default:
     NOTE_EVENT; }
   
@@ -676,21 +686,36 @@ posix_request_interrupts(s48_value sch_signum)
  * and remove it from the saved_action array.
  */
 
+static void
+cancel_interrupt_request(int signum)
+{
+  struct sigaction *	old = saved_actions[signum];
+
+  if (old != NULL)
+    {
+      
+      if (sigaction(signum, old, (struct sigaction *) NULL) != 0)
+	s48_raise_os_error(errno);
+      
+      free(old);
+      saved_actions[signum] = NULL; 
+    }
+}
+
 s48_value
 posix_cancel_interrupt_request(s48_value sch_signum)
 {
-  int			signum = s48_extract_fixnum(sch_signum);
-  struct sigaction *	old = saved_actions[signum];
-
-  if (old != NULL) {
-    
-    if (sigaction(signum, old, (struct sigaction *) NULL) != 0)
-      s48_raise_os_error(errno);
-    
-    free(old);
-    saved_actions[signum] = NULL; }
-    
+  cancel_interrupt_request(s48_extract_fixnum(sch_signum));
   return S48_UNSPECIFIC;
 }
 
-
+static void
+cancel_interrupt_requests(void)
+{
+  int signum = 0;
+  while (signum <= MAX_SIGNAL)
+    {
+      cancel_interrupt_request(signum);
+      ++signum;
+    }
+}
