@@ -28,28 +28,18 @@ extern s48_value	s48_extended_vm(long, s48_value),
  */
 
 extern long		s48_call_native_procedure(long, long),
-			s48_invoke_native_continuation(long);
-
-/*
- * The following are hand-written macro versions of procedures
- * in scheme48heap.c.
- */
-
-#define	AVAILABLEp(cells)	(s48_ShpS + ((cells)<<2) < s48_SlimitS)
-
-static char	*_HHallocate_temp;
-
-#define	ALLOCATE_SPACE(type, len)		\
-		(_HHallocate_temp = s48_ShpS,	\
-			s48_ShpS += ((len)+3) & ~3,	\
-			_HHallocate_temp)
-
+			s48_invoke_native_continuation(long, long);
 
 #define	GET_PROPOSAL_LOCK()	((void)0)
 #define	RELEASE_PROPOSAL_LOCK()	((void)0)
 #define SHARED_REF(x)		(x)
 #define SHARED_SETB(x, v)	((x) = (v))
 
+char *s48_get_os_string_encoding(void);
+
+extern void s48_raise_argument_type_error(s48_value);
+extern void s48_raise_range_error(s48_value, s48_value, s48_value);
+
 /*
  * We rename these to avoid name clashes.
  */
