@@ -1,4 +1,4 @@
-/* Copyright (c) 1993-2005 by Richard Kelsey and Jonathan Rees.
+/* Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees.
    See file COPYING. */
 
 /*
@@ -8,6 +8,7 @@
 #define _XOPEN_SOURCE_EXTENDED 1		/* AIX wants this to be 1 */
 #include "sysdep.h"
 #include <stdio.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/param.h>
@@ -35,19 +36,23 @@ static s48_value	s48_socket(s48_value udp_p, s48_value input_p),
 			s48_listen(s48_value socket_channel),
 			s48_accept(s48_value socket_channel),
 			s48_connect(s48_value socket_channel,
-				    s48_value machine,
+				    s48_value address,
 				    s48_value port,
 				    s48_value retry_p),
                         s48_dup_socket_channel(s48_value socket_fd),
 			s48_close_socket_half(s48_value socket_channel,
 					      s48_value input_p),
+                        s48_get_host_by_name(s48_value machine),
+                        s48_get_host_by_name_result(s48_value event_uid),
+                        s48_get_host_by_address(s48_value address),
+                        s48_get_host_by_address_result(s48_value event_uid),
 			s48_get_host_name(void),
 			s48_udp_send(s48_value channel,
 				     s48_value udp_address,
 				     s48_value buffer,
 				     s48_value length),
 			s48_udp_receive(s48_value channel, s48_value message),
-			s48_lookup_udp_address(s48_value name, s48_value port);
+			s48_lookup_udp_address(s48_value address, s48_value port);
 
 /* Forward declaration. */
 static s48_value dup_socket_channel(int socket_fd);
@@ -85,6 +90,10 @@ s48_init_socket(void)
   S48_EXPORT_FUNCTION(s48_connect);
   S48_EXPORT_FUNCTION(s48_dup_socket_channel);
   S48_EXPORT_FUNCTION(s48_close_socket_half);
+  S48_EXPORT_FUNCTION(s48_get_host_by_name);
+  S48_EXPORT_FUNCTION(s48_get_host_by_name_result);
+  S48_EXPORT_FUNCTION(s48_get_host_by_address);
+  S48_EXPORT_FUNCTION(s48_get_host_by_address_result);
   S48_EXPORT_FUNCTION(s48_get_host_name);
   
   S48_EXPORT_FUNCTION(s48_udp_send);
@@ -111,19 +120,30 @@ s48_socket(s48_value udp_p, s48_value input_p)
     		mode,
     		status;
   s48_value	channel;
+  int on = 1;
 
-  RETRY_OR_RAISE_NEG(fd, socket(AF_INET,
+  RETRY_OR_RAISE_NEG(fd, socket(PF_INET,
 				(udp_p == S48_FALSE) ?
 				  SOCK_STREAM :
 				  SOCK_DGRAM,
 				0));
   RETRY_OR_RAISE_NEG(status, fcntl(fd, F_SETFL, O_NONBLOCK));
 
+  /*
+   * If we don't do this, we may get "Address already in use" if we
+   * try to do anything on the same port too soon.
+   * Alan Bawden says this is OK:
+   * http://news.gmane.org/gmane.lisp.scheme.scheme48/cutoff=1672
+   */
+  RETRY_OR_RAISE_NEG(status,
+		     setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
+				&on, sizeof(on)));
+
   mode = (input_p == S48_FALSE) ?
     S48_CHANNEL_STATUS_SPECIAL_OUTPUT :
     S48_CHANNEL_STATUS_SPECIAL_INPUT;
 
-  channel = s48_add_channel(mode, s48_enter_string("socket"), fd);
+  channel = s48_add_channel(mode, s48_enter_string_latin_1("socket"), fd);
 
   if (!S48_CHANNEL_P(channel)) {
     ps_close_fd(fd);		/* retries if interrupted */
@@ -254,7 +274,7 @@ s48_accept(s48_value channel)
     RETRY_OR_RAISE_NEG(status, fcntl(connect_fd, F_SETFL, O_NONBLOCK));
 
     input_channel = s48_add_channel(S48_CHANNEL_STATUS_INPUT,
-				    s48_enter_string("socket connection"),
+				    s48_enter_string_latin_1("socket connection"),
 				    connect_fd);
 
     if (!S48_CHANNEL_P(input_channel)) {
@@ -278,6 +298,256 @@ s48_accept(s48_value channel)
   return S48_FALSE;
 }
 
+static s48_value
+call_gethostbyaddr(s48_value sch_addr)
+{
+  char *addr = s48_extract_byte_vector(sch_addr);
+  char		*hostname;
+  struct hostent *hostdata;
+  
+  hostdata = gethostbyaddr(addr,
+			   sizeof(struct in_addr),
+			   AF_INET);
+  if (hostdata == NULL)
+    hostname = inet_ntoa(*((struct in_addr *) addr));
+  else
+    hostname = hostdata->h_name;
+  
+  return s48_enter_string_latin_1(hostname);
+}
+
+#ifdef HAVE_PTHREAD_H
+#define S48_MAX_HOST_NAME_LENGTH 1024
+
+struct gethostbyname_handshake
+{
+  long event_uid;
+  char host_name[S48_MAX_HOST_NAME_LENGTH + 1]; /* is enough */
+  int errno_val;
+  int host_length;
+  char* host_addr;
+};
+
+/*
+ * Only one call can be active at any given time:
+ * We expect mutual exclusion at the Scheme end
+ */
+
+static void
+gethostbyname_thread(void *void_handshake)
+{
+  struct gethostbyname_handshake *handshake = void_handshake;
+  static struct hostent *host; /* will be thread-local */
+
+  host = gethostbyname(handshake->host_name);
+  if (host == NULL)
+    handshake->errno_val = h_errno;
+  else
+    {
+      handshake->host_length = host->h_length;
+      handshake->host_addr = malloc(handshake->host_length);
+      if (handshake->host_addr == NULL)
+	handshake->errno_val = ENOMEM;
+      else
+	handshake->errno_val = 0;
+      
+      memcpy(handshake->host_addr, host->h_addr, handshake->host_length);
+    }
+  s48_note_external_event((long) handshake->event_uid);
+}
+
+static s48_value
+s48_get_host_by_name(s48_value machine)
+{
+  pthread_t t;
+  struct gethostbyname_handshake* handshake;
+
+  int machine_length = S48_BYTE_VECTOR_LENGTH(machine);
+  if (machine_length > S48_MAX_HOST_NAME_LENGTH)
+    s48_raise_argument_type_error(machine);
+
+  handshake = malloc(sizeof(struct gethostbyname_handshake));
+  if (handshake == NULL)
+    s48_raise_out_of_memory_error();
+
+  memcpy(handshake->host_name, s48_extract_byte_vector(machine), machine_length);
+  handshake->host_name[machine_length] = '\0';
+  
+  handshake->event_uid = s48_external_event_uid();
+  if (pthread_create(&t, NULL, gethostbyname_thread, (void*) handshake))
+    {
+      static struct hostent *host;
+      s48_unregister_external_event_uid(handshake->event_uid);
+      host = gethostbyname(handshake->host_name);
+      free(handshake);
+      if (host == NULL)
+	s48_raise_os_error(h_errno);
+      return s48_enter_byte_vector(host->h_addr, host->h_length);
+    }
+  else
+    {
+      s48_value sch_event_uid = S48_UNSPECIFIC;
+      s48_value sch_handshake = S48_UNSPECIFIC;
+      s48_value sch_result;
+      S48_DECLARE_GC_PROTECT(2);
+
+      pthread_detach(t);
+
+      S48_GC_PROTECT_2(sch_event_uid, sch_handshake);
+      sch_event_uid = s48_enter_integer(handshake->event_uid);
+      sch_handshake = s48_enter_pointer(handshake);
+
+      sch_result = s48_cons(sch_event_uid, sch_handshake);
+      S48_GC_UNPROTECT();
+      return sch_result;
+    }
+}
+
+static s48_value
+s48_get_host_by_name_result(s48_value sch_handshake)
+{
+  struct gethostbyname_handshake* handshake
+    = s48_extract_pointer(sch_handshake);
+
+  s48_unregister_external_event_uid(handshake->event_uid);
+
+  if (handshake->errno_val)
+    {
+      int errno_val = handshake->errno_val;
+      free(handshake);
+      s48_raise_os_error(errno_val);
+    }
+  else
+    {
+      s48_value sch_result
+	= s48_enter_byte_vector(handshake->host_addr, handshake->host_length);
+      free(handshake->host_addr);
+      free(handshake);
+      return sch_result;
+    }
+}
+
+/*
+ * Get the name of the `sch_addr''s host.  If we can't get the real
+ * host name we use the numbers-and-dots representation of the
+ * address.
+ */
+
+struct gethostbyaddr_handshake
+{
+  long event_uid;
+  struct in_addr address;
+  char host_name[S48_MAX_HOST_NAME_LENGTH + 1];
+};
+
+static void
+gethostbyaddr_thread(void *void_handshake)
+{
+  struct gethostbyaddr_handshake *handshake = void_handshake;
+  struct in_addr addr;
+  struct hostent *hostdata;
+  
+  hostdata = gethostbyaddr(&(handshake->address),
+			   sizeof(struct in_addr),
+			   AF_INET);
+  if (hostdata == NULL)
+    strcpy(handshake->host_name,
+	   inet_ntoa(*((struct in_addr *) &(handshake->address))));
+  else
+    strcpy(handshake->host_name, hostdata->h_name);
+
+  s48_note_external_event((long) handshake->event_uid);
+}
+ 
+static s48_value
+s48_get_host_by_address(s48_value sch_addr)
+{
+  pthread_t t;
+  struct gethostbyaddr_handshake* handshake;
+  
+  handshake = malloc(sizeof(struct gethostbyaddr_handshake));
+  if (handshake == NULL)
+    s48_raise_out_of_memory_error();
+
+  memcpy(&(handshake->address),
+	 (char *) s48_extract_byte_vector(sch_addr), sizeof(struct in_addr));
+
+  handshake->event_uid = s48_external_event_uid();
+  if (pthread_create(&t, NULL, gethostbyaddr_thread, (void*) handshake))
+    {
+      s48_unregister_external_event_uid(handshake->event_uid);
+      free(handshake);
+      return call_gethostbyaddr(sch_addr);
+    }
+  else
+    {
+      s48_value sch_event_uid = S48_UNSPECIFIC;
+      s48_value sch_handshake = S48_UNSPECIFIC;
+      s48_value sch_result;
+      S48_DECLARE_GC_PROTECT(2);
+
+      pthread_detach(t);
+
+      S48_GC_PROTECT_2(sch_event_uid, sch_handshake);
+      sch_event_uid = s48_enter_integer(handshake->event_uid);
+      sch_handshake = s48_enter_pointer(handshake);
+
+      sch_result = s48_cons(sch_event_uid, sch_handshake);
+      S48_GC_UNPROTECT();
+      return sch_result;
+    }
+}
+	 
+static s48_value 
+s48_get_host_by_address_result(s48_value sch_handshake)
+{
+  struct gethostbyaddr_handshake* handshake
+    = s48_extract_pointer(sch_handshake);
+  s48_value sch_result;
+
+  s48_unregister_external_event_uid(handshake->event_uid);
+  sch_result = s48_enter_string_latin_1(handshake->host_name);
+  free(handshake);
+  return sch_result;
+}
+#else
+static s48_value
+s48_get_host_by_name(s48_value machine)
+{
+  struct hostent *host;
+  char* machine_name = s48_extract_byte_vector(machine);
+  RETRY_NULL(host, gethostbyname(machine_name));
+  if (host == NULL)
+    s48_raise_os_error(h_errno);
+  return s48_enter_byte_vector(host->host_addr, host->host_length);
+}
+
+static s48_value
+s48_get_host_by_name_result(void)
+{
+  /* won't be used here */
+}
+
+/*
+ * Get the name of the `sch_addr''s host.  If we can't get the real
+ * host name we use the numbers-and-dots representation of the
+ * address.
+ */
+ 
+static s48_value
+s48_get_host_by_address(s48_value sch_addr)
+{
+  return call_gethostbyaddr(sch_addr);
+}
+
+static s48_value
+s48_get_host_by_address_result(void)
+{
+  /* won't be used here */
+}
+#endif
+
+
 /*
  * Given an internet-domain stream socket, a machine name and a port number,
  * connect the socket to that machine/port.
@@ -290,41 +560,34 @@ s48_accept(s48_value channel)
 
 static s48_value
 s48_connect(s48_value channel,
-	    s48_value machine,
+	    s48_value sch_address,
 	    s48_value port,
 	    s48_value retry_p)
 {
   int			socket_fd,
     			port_number;
-  char			*machine_name;
-  struct hostent	*host;
   struct sockaddr_in	address;
 
   S48_CHECK_CHANNEL(channel);
   socket_fd = S48_UNSAFE_EXTRACT_FIXNUM(S48_UNSAFE_CHANNEL_OS_INDEX(channel));
 
-  S48_CHECK_STRING(machine);
-  machine_name = S48_UNSAFE_EXTRACT_STRING(machine);
-  
   S48_CHECK_FIXNUM(port);
   port_number = S48_UNSAFE_EXTRACT_FIXNUM(port);
   
   /*
-   * Get the host and initialize `address'.
+   * Initialize `address'.
    */
 
-  RETRY_NULL(host, gethostbyname(machine_name));
-  if (host == NULL)
-    s48_raise_os_error(h_errno);
-  
   memset((void *)&address, 0, sizeof(address));
-  address.sin_family = host->h_addrtype;
+  address.sin_family = AF_INET;
 
-  if (host->h_length > sizeof(address.sin_addr))
-    s48_raise_range_error(s48_enter_fixnum(host->h_length),
+  if (S48_BYTE_VECTOR_LENGTH(sch_address) > sizeof(address.sin_addr))
+    s48_raise_range_error(s48_enter_fixnum(S48_BYTE_VECTOR_LENGTH(sch_address)),
 			  S48_UNSAFE_ENTER_FIXNUM(0),
 			  s48_enter_fixnum(sizeof(address.sin_addr)));
-  memcpy((void *)&address.sin_addr, (void *)host->h_addr, host->h_length);
+  memcpy((void *)&address.sin_addr,
+	 (void *)s48_extract_byte_vector(sch_address),
+	 S48_BYTE_VECTOR_LENGTH(sch_address));
   address.sin_port = htons(port_number);
 
   /*
@@ -389,7 +652,7 @@ dup_socket_channel(int socket_fd)
   RETRY_OR_RAISE_NEG(output_fd, dup(socket_fd));
 
   output_channel = s48_add_channel(S48_CHANNEL_STATUS_OUTPUT,
-				   s48_enter_string("socket connection"),
+				   s48_enter_string_latin_1("socket connection"),
 				   output_fd);
   
   if (!S48_CHANNEL_P(output_channel)) {
@@ -435,7 +698,7 @@ s48_get_host_name(void)
 
   RETRY_OR_RAISE_NEG(status, gethostname(mbuff, sizeof(mbuff)));
 
-  return s48_enter_string(mbuff);
+  return s48_enter_string_latin_1(mbuff);
 }
 
 /*
@@ -578,19 +841,10 @@ address_connection(struct sockaddr_in *addr)
 }
 
 static s48_value
-s48_lookup_udp_address(s48_value name, s48_value port)
+s48_lookup_udp_address(s48_value address, s48_value port)
 {
-  struct hostent *	host = gethostbyname(s48_extract_string(name));
-  struct in_addr	address;
-  
-  if (host == NULL ||
-      host->h_addrtype != AF_INET ||	/* could happen, I suppose */
-      host->h_addr_list[0] == NULL)
-    s48_raise_os_error(errno);
-    
-  address = *((struct in_addr *) host->h_addr_list[0]);
-
-  return lookup_connection(address, s48_extract_fixnum(port));
+  return lookup_connection(*((struct in_addr *) s48_extract_byte_vector(address)),
+			   s48_extract_fixnum(port));
 }
 
 /*
@@ -640,16 +894,13 @@ add_new_connection(int index, struct in_addr address, unsigned long port)
 
   S48_GC_PROTECT_1(udp_address);
 
-  sch_address = S48_MAKE_VALUE(struct in_addr);		/* may GC */
+  sch_address
+    = s48_enter_byte_vector((char*) (&address), sizeof(struct in_addr)); /* may GC */
   
-  S48_UNSAFE_EXTRACT_VALUE(sch_address, struct in_addr) = address;
-
   S48_UNSAFE_RECORD_SET(udp_address, 0, sch_address);
   S48_UNSAFE_RECORD_SET(udp_address, 1, S48_UNSAFE_ENTER_FIXNUM(port));
   
-  name = get_hostname(address);				/* may GC */
-
-  S48_UNSAFE_RECORD_SET(udp_address, 2, name);
+  S48_UNSAFE_RECORD_SET(udp_address, 2, S48_FALSE);
 
   S48_UNSAFE_VECTOR_SET(connections, index, udp_address);
   connection_count += 1;
@@ -661,26 +912,6 @@ add_new_connection(int index, struct in_addr address, unsigned long port)
   return udp_address;
 }
 
-/*
- * Get the name of the `addr''s host.  If we can't get the real host name
- * we use the numbers-and-dots representation of the address.
- */
- 
-static s48_value
-get_hostname(struct in_addr addr)
-{
-  char		*hostname;
-  struct hostent *hostdata;
-  
-  hostdata = gethostbyaddr((char *) &addr, sizeof(struct in_addr), AF_INET);
-  if (hostdata == NULL)
-    hostname = inet_ntoa(addr);
-  else
-    hostname = hostdata->h_name;
-  
-  return s48_enter_string(hostname);
-}
-
 /*
  * Double the size of the hash table and rehash all connections into it.
  */
