@@ -39,19 +39,17 @@
     (compile-constant (cadr (node-form node)) depth frame cont)))
 
 (define (compile-constant obj depth frame cont)
-  (deliver-value (cond ((ignore-values-cont? cont)
-			empty-segment)			;+++ dead code
-		       ((eq? obj #f)
-			; +++ hack for bootstrap from Schemes that don't
-			; distinguish #f/()
-			(instruction (enum op false)))
-		       ((small-integer? obj)
-			(integer-literal-instruction obj))
-		       (else
-			(stack-indirect-instruction
-			  (template-offset frame depth)
-			  (literal->index frame obj))))
-		 cont))
+  (deliver-constant-value (cond ((eq? obj #f)
+				 ;; +++ hack for bootstrap from Schemes that don't
+				 ;; distinguish #f/()
+				 (instruction (enum op false)))
+				((small-integer? obj)
+				 (integer-literal-instruction obj))
+				(else
+				 (stack-indirect-instruction
+				  (template-offset frame depth)
+				  (literal->index frame obj))))
+			  cont))
 
 (define (small-integer? obj)
   (and (integer? obj)
@@ -67,13 +65,19 @@
 
 (define-compilator 'unspecific (proc () unspecific-type)
   (lambda (node depth frame cont)
-    (deliver-value (instruction (enum op unspecific))
-		   cont)))
+    (deliver-constant-value (instruction (enum op unspecific))
+			    cont)))
 
 (define-compilator 'unassigned (proc () unspecific-type)
   (lambda (node depth frame cont)
-    (deliver-value (instruction (enum op unassigned))
-		   cont)))
+    (deliver-constant-value (instruction (enum op unassigned))
+			    cont)))
+
+(define (deliver-constant-value segment cont)
+  (deliver-value (if (ignore-values-cont? cont)
+		     empty-segment
+		     segment)
+		 cont))
 
 ;----------------------------------------------------------------
 ; Variable reference
@@ -104,10 +108,6 @@
 	  (else
 	   (error "variable has too many indirections" name binding)))))
 
-(define (index->offset index depth)
-  (- (- depth 1)
-     index))
-
 ;----------------------------------------------------------------
 ; Hacked versions of the above for peephole optimization of pushes.
 ; This tries to compile NODE with an implicit push instruction preceeding
@@ -451,7 +451,7 @@
     (values (instruction (enum op push-false))
 	    (+ depth 1)
 	    label
-	    (sequentially (continuation-data #f depth)
+	    (sequentially (continuation-data #f depth (template-offset frame depth))
 			  (attach-label label protocol)))))
 
 (define (fixup-source node destination)
@@ -763,19 +763,6 @@
   (lambda (node depth frame cont)
     (compile (caddr (node-form node)) depth frame cont)))
 
-; Node predicates and operators.
-
-(define lambda-node?      (node-predicate 'lambda      syntax-type))
-(define flat-lambda-node? (node-predicate 'flat-lambda syntax-type))
-(define name-node?        (node-predicate 'name        'leaf))
-(define literal-node?     (node-predicate 'literal     'leaf))
-(define quote-node?       (node-predicate 'quote       syntax-type))
-
-(define operator/lambda     (get-operator 'lambda syntax-type))
-(define operator/set!	    (get-operator 'set!   syntax-type))
-(define operator/call	    (get-operator 'call   'internal))
-(define operator/begin      (get-operator 'begin  syntax-type))
-
 ; Should this be in its own spot?
 
 (define-syntax receive
