@@ -1,5 +1,5 @@
 ; -*- Mode: Scheme; Syntax: Scheme; Package: Scheme; -*-
-; Copyright (c) 1993-2005 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
 
 ; Compiling primitive procedures and calls to them.
 
@@ -29,8 +29,9 @@
 			       name	; name of primop
 			       (or maybe-nargs 0)
 					; nargs (needed if template used)
+			       maybe-nargs ; need template if nargs
 			       #f	; no env
-			       maybe-nargs))) ; need template if nargs
+			       #f)))    ; no closure
 	(segment->template (proc frame) frame)))))
 
 ; --------------------
@@ -53,9 +54,9 @@
     (let ((arg-specs (vector-ref opcode-arg-specs opcode)))
       (sequentially (if (pair? arg-specs)
                         (sequentially
-                         (lambda-protocol (car arg-specs) #f #f)
+                         (lambda-protocol (car arg-specs) #f #f #f)
                          (instruction (enum op pop)))
-                        (lambda-protocol 0 #f #f))
+                        (lambda-protocol 0 #f #f #f))
                     (instruction opcode)
                     (instruction (enum op return))))))
 
@@ -111,19 +112,18 @@
 	    exact-integer-type))
     ((char=? char<?)
      ,(proc (char-type char-type) boolean-type))
-    (char->ascii
-     ,(proc (char-type) exact-integer-type))
-    (ascii->char
-     ,(proc (exact-integer-type) char-type))
     (string=?
      ,(proc (string-type string-type) boolean-type))
     (open-channel
      ;; Can return #f
-     ,(proc (string-type exact-integer-type boolean-type) value-type))
+     ,(proc (string-type value-type exact-integer-type boolean-type) value-type))
     (cons
      ,(proc (value-type value-type) pair-type))
     (intern
-     ,(proc (string-type) symbol-type))))
+     ,(proc (string-type) symbol-type))
+    (make-weak-pointer
+     ,(proc (value-type) value-type))))
+
 
 ; Can't do I/O until the meta-types interface exports input-port-type and
 ; output-port-type.
@@ -145,7 +145,8 @@
           ((not (number? (car arg-specs))))
 	  ((memq name '(+ * - / = < > <= >=
 			bitwise-ior bitwise-xor bitwise-and
-			make-string closed-apply)))
+			make-string closed-apply
+			encode-char/us-ascii)))
           (else
            (let ((type (get-primop-type name (car arg-specs))))
              (define-compiler-primitive name type
@@ -175,17 +176,13 @@
 
 (define (simple-closed-compilator nargs segment)
   (lambda (frame)
-    (sequentially (lambda-protocol nargs #f #f)
+    (sequentially (lambda-protocol nargs #f #f #f)
                   (if (< 0 nargs)
                       (instruction (enum op pop))
                       empty-segment)
                   segment
                   (instruction (enum op return)))))
 
-(define (symbol-append . syms)
-  (string->symbol (apply string-append
-                         (map symbol->string syms))))
-
 (define (define-stob-predicate name stob-name)
   (define-simple-primitive name
     (proc (value-type) boolean-type)
@@ -202,15 +199,11 @@
   (define-compiler-primitive 'make-double (proc () :double)
     (lambda (node depth frame cont)
       (deliver-value
-        (sequentially
-	  (stack-indirect-instruction (template-offset frame depth)
-				      (literal->index frame 0))
-	  (instruction (enum op push))		; leaves value in *val*
-	  (instruction (enum op make-stored-object) 2 (enum stob double)))
+       (instruction (enum op make-double))
 	cont))
     (cons 0
 	  (lambda (frame)
-	    (sequentially (lambda-protocol 0 #t #f)
+	    (sequentially (lambda-protocol 0 #t #f #f)
 			  (instruction (enum op stack-indirect)
 				       (template-offset frame 1) ; template
 				       (literal->index frame 0))
@@ -400,7 +393,7 @@
 		      (call-instruction 1 label)	; one argument
 		      after))))
   (lambda (frame)
-    (sequentially (lambda-protocol 1 #f #f)
+    (sequentially (lambda-protocol 1 #f #f #f)
                   (instruction (enum op current-cont))
 		  (instruction (enum op push))
 		  (instruction (enum op stack-ref) 1)
@@ -459,7 +452,7 @@
     (lambda (frame)
       (receive (before depth label after)
 	  (push-continuation-no-protocol 2 frame #f (plain-fall-through-cont))
-	(sequentially (lambda-protocol 2 #f #f)
+	(sequentially (lambda-protocol 2 #f #f #f)
 		      (instruction (enum op stack-ref+push) 1)
 		      (instruction (enum op false))
 		      (instruction (enum op stack-set!) 2)
@@ -625,9 +618,9 @@
   (n-ary-constructor 'vector vector-type (enum stob vector))
   (n-ary-constructor 'record #f (enum stob record)))
 
-; READ-CHAR, PEEK-CHAR and WRITE-CHAR
+; READ-BYTE, PEEK-BYTE and WRITE-BYTE
 
-(let ((define-char-io
+(let ((define-byte/char-io
 	(lambda (id opcode type)
 	  (define-compiler-primitive id
 	    type
@@ -651,14 +644,20 @@
 			      (instruction (enum op return)))
 	       empty-segment
 	       empty-segment))))))
-  (define-char-io 'read-char
+  (define-byte/char-io 'read-byte
+    (enum op read-byte)
+    (proc (&opt value-type) value-type))
+  (define-byte/char-io 'peek-byte
+    (enum op peek-byte)
+    (proc (&opt value-type) value-type))
+  (define-byte/char-io 'read-char
     (enum op read-char)
     (proc (&opt value-type) value-type))
-  (define-char-io 'peek-char
+  (define-byte/char-io 'peek-char
     (enum op peek-char)
     (proc (&opt value-type) value-type)))
 
-(let ((define-char-io
+(let ((define-byte/char-io
 	(lambda (id opcode type)
 	  (define-compiler-primitive id
 	    type
@@ -684,7 +683,10 @@
 			      (instruction opcode 0)
 			      (instruction (enum op return)))
                 empty-segment))))))
-  (define-char-io 'write-char
+  (define-byte/char-io 'write-byte
+    (enum op write-byte)
+    (proc (integer-type &opt value-type) unspecific-type))
+  (define-byte/char-io 'write-char
     (enum op write-char)
     (proc (char-type &opt value-type) unspecific-type)))
 
@@ -919,5 +921,50 @@
     (instruction (enum op make-string))
     (make-node op/literal #\?)
     (sequentially (integer-literal-instruction (char->ascii #\?))
-		  (instruction (enum op ascii->char)))
+		  (instruction (enum op scalar-value->char)))
     (proc (number-type &opt char-type) string-type)))
+
+; Text encoding/decoding
+
+; These return multiple values, which is why this is more work.
+
+(let ((define-encode/decode
+	(lambda (name type arg-count retval-count
+		      regular bang)
+	  (let ((depth-inc (max (- arg-count 1) retval-count)))
+	    (define-compiler-primitive name type
+	      (lambda (node depth frame cont)
+		(depth-check! frame (+ depth depth-inc))
+		(let ((args (cdr (node-form node))))
+		  (cond
+		   ((return-cont? cont)
+		    (sequentially (push-all-but-last args depth frame node)
+				  (instruction regular)))
+		   ((ignore-values-cont? cont)
+		    (sequentially (push-all-but-last args depth frame node)
+				  (instruction bang)))
+		   ((fall-through-cont? cont)
+		    (generate-trap depth
+				   frame
+				   cont
+				   (string-append "returning " 
+						  (number->string retval-count)
+						  " arguments where one is expected")
+				   (schemify node)))
+		   (else
+		    (error "unknown compiler continuation" (enumerand->name regular op) cont)))))
+	      (direct-closed-compilator regular))))))
+
+  (define-encode/decode 'encode-char 
+    (proc (exact-integer-type char-type value-type exact-integer-type exact-integer-type)
+	  (make-some-values-type (list boolean-type value-type)))
+    5 2
+    (enum op encode-char) (enum op encode-char!))
+
+  (define-encode/decode 'decode-char 
+    (proc (exact-integer-type value-type exact-integer-type exact-integer-type)
+	  (make-some-values-type (list value-type value-type)))
+    4 2
+    (enum op decode-char) (enum op decode-char!)))
+
+  
