@@ -1,4 +1,4 @@
-; Copyright (c) 1993-2005 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
 
 ; Threads.
 
@@ -641,13 +641,15 @@
 ; Enqueue a RUNNABLE event for THREAD's scheduler.
 
 (define (make-ready thread . args)
-  (clear-thread-cell! thread)
-  (set-thread-arguments! thread args)
-  (if (thread-scheduler thread)
-      (schedule-event (thread-scheduler thread)
-		      (enum event-type runnable)
-		      thread)
-      (error "MAKE-READY thread has no scheduler" thread)))
+  (if (thread-cell thread)
+      (begin
+	(clear-thread-cell! thread)
+	(set-thread-arguments! thread args)
+	(if (thread-scheduler thread)
+	    (schedule-event (thread-scheduler thread)
+			    (enum event-type runnable)
+			    thread)
+	    (error "MAKE-READY thread has no scheduler" thread)))))
 
 (define (clear-thread-cell! thread)
   (let ((cell (thread-cell thread)))
