@@ -1,4 +1,4 @@
-; Copyright (c) 1993-2005 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
 
 (define-interface vm-utilities-interface
   (export adjoin-bits low-bits high-bits unsigned-high-bits
@@ -9,6 +9,13 @@
 	  error?
 	  write-error-string write-error-integer write-error-newline
 	  error-message
+
+	  ;; needed in portable-read-image and measurements
+	  write-out-string
+	  write-out-integer
+	  write-out-newline
+	  display-message  ;with newline
+	  display-integer  ;with newline
 	  ))
 
 ; Memory
@@ -64,19 +71,25 @@
 	  valid-index?
 	  	
 	  immutable? make-immutable!
+	  make-weak-pointer
 	  ))
 
 (define-interface allocation-interface
   (export s48-make-available+gc
 	  s48-allocate-small
+	  s48-allocate-weak+gc
 	  s48-allocate-traced+gc
 	  s48-allocate-untraced+gc
 	  s48-allocate-stob
+	  
+	  s48-forbid-gc!
+	  s48-allow-gc!
           ))
 
 (define-interface heap-interface
   (export s48-available
 	  s48-heap-size
+	  s48-max-heap-size
 
 	  s48-gather-objects
 
@@ -85,26 +98,14 @@
 
 	  s48-write-barrier
 
-	  ; for write-image
-	  s48-heap-pointer
-	  s48-oldspace-begin
-	  s48-oldspace-end
-	  s48-heap-begin
-
-	  s48-*hp*	; to keep these from being `static' in C, as
-	  s48-*limit*	; references are introduced elsewhere via a C macro
-
 	  ; for debugging
 	  s48-check-heap
+	  s48-stob-in-heap?
 	  ))
 
+
 (define heap-gc-util-interface
-  (export swap-spaces
-	  heap-limit
-	  set-heap-pointer!
-	  heap-pointer
-	  walk-impure-areas
-	  allocate
+  (export walk-impure-areas
 	  bytes-available?))
 
 (define-interface gc-roots-interface
@@ -135,21 +136,30 @@
 	  s48-extant?
 
 	  s48-gc-count
+          s48-gc-run-time
 	  ))
 
-(define external-opcodes-interface
-  (export ; called from outside
+(define shared-bindings-access-interface
+  (export ;; for writing images
+          s48-imported-bindings
+	  s48-exported-bindings))
+  
+(define shared-bindings-interface
+  (export ;; called from outside	
           s48-define-exported-binding
 	  s48-get-imported-binding
-	  s48-setup-external-exception
-	  s48-resetup-external-exception
 
-	  ; called on startup
+	  ;; called on startup
 	  install-shared-bindings!+gc
 	  
-	  ; for writing images
-	  s48-imported-bindings
-	  s48-exported-bindings
+	  ;; for external events
+	  get-imported-binding
+
+	  ;; for the primitives
+	  undefine-shared-binding!
+	  lookup-imported-binding lookup-exported-binding
+	  shared-binding-undefined? for-each-imported-binding
+	  shared-binding-size
 	  ))
 
 (define-interface external-gc-roots-interface
@@ -161,41 +171,16 @@
 	  s48-unregister-gc-root!
 	  s48-reset-external-roots!))
 
-(define-interface read-image-interface
-  (export s48-read-image
-
-	  s48-relocate-all
-	  s48-reverse-byte-order!
-
-	  s48-startup-procedure
-	  s48-initial-symbols
-	  s48-initial-imported-bindings
-	  s48-initial-exported-bindings
-	  s48-resumer-records
-
-	  s48-initialization-complete!
-	  s48-initializing-gc-root
-
-	  s48-set-image-values!))
-
-(define-interface image-gc-interface
-  (export begin-making-image
-	  trace-image-value
-	  make-image
-	  restore-heap
-
-	  image-start
-	  image-pointer
-
-	  make-image-symbol-table
-	  copy-exported-bindings
-	  clean-imported-bindings
-	  mark-image-channels-closed!
-	  find-resumer-records))
-
 (define-interface heap-init-interface
-  (export s48-initialize-heap
-	  s48-register-static-areas))
+  (export s48-register-static-areas
+
+	  ;; two-space/BIBOP
+	  s48-initialize-heap
+
+	  ;;BIBOP
+	  s48-initialize-image-areas
+	  s48-check-heap-size!
+	  ))
 
 (define-interface string-table-interface
   (export hash-table-size
@@ -207,11 +192,16 @@
 	  table-walker table-while-walker
 	  table-tracer
 	  table-cleaner
-	  table-relocator
 
+	  relocate-table
 	  value->link
 	  link->value))
 
+(define-interface text-encodings-interface
+  (export encode-scalar-value
+	  decode-scalar-value
+	  (text-encoding-option :syntax )))
+
 (define-interface struct-interface
   (export vm-pair? vm-pair-size vm-cons vm-car vm-set-car! vm-cdr vm-set-cdr!
 	  vm-symbol? vm-symbol-size vm-make-symbol vm-symbol->string
@@ -242,7 +232,10 @@
 	  port-buffer set-port-buffer!
 	  port-index set-port-index!
 	  port-limit set-port-limit!
+	  port-text-codec-spec set-port-text-codec-spec!
+	  port-crlf? set-port-crlf?!
 	  port-lock
+	  port-pending-cr?  set-port-pending-cr?!
 	  port-pending-eof? set-port-pending-eof?!
    
 	  vm-vector? vm-vector-size vm-make-vector vm-vector-length
@@ -275,10 +268,15 @@
 	  template-ref template-set!
 	  template-code template-byte-code template-name
 
-	  vm-string? vm-string-size vm-make-string vm-string-length
+	  vm-make-string vm-make-string+gc
+	  vm-string? vm-string-size vm-string-length
 	  vm-string-ref vm-string-set!
-	  enter-string enter-string+gc extract-string
+	  enter-string enter-string+gc enter-string+gc-n extract-low-string
 	  vm-string=? vm-string-hash
+	  copy-vm-string-chars!
+	  write-vm-string
+	  copy-string-to-vm-string/latin-1!
+	  copy-vm-string-to-string/latin-1!
 
 	  ratnum?
 
@@ -335,6 +333,7 @@
 	  make-continuation-on-stack
 	  push-continuation!
 	  push-adlib-continuation!
+	  set-cont-to-stack!
 	  push-exception-continuation!
 	  push-native-exception-continuation!
 	  pop-continuation-from-stack
@@ -382,6 +381,11 @@
 	  cheap-time
 	  schedule-interrupt
 
+	  dequeue-external-event!
+
+	  get-os-string-encoding
+	  host-architecture
+
  	  external-bignum-make-cached-constants
  	  external-bignum-add
  	  external-bignum-subtract
@@ -400,11 +404,13 @@
  	  external-bignum-bitwise-ior
  	  external-bignum-bitwise-xor
  	  external-bignum-from-long
+	  external-bignum-from-unsigned-long
  	  external-bignum->long
  	  external-bignum-fits-in-word?
 
 	  s48-call-native-procedure
 	  s48-invoke-native-continuation
+          s48-jump-native
 	  s48-native-return
 
 	  get-proposal-lock!
@@ -412,6 +418,9 @@
 
 	  shared-ref
 	  (shared-set! :syntax)
+
+	  raise-argument-type-error
+	  raise-range-error
 	  ))
 
 (define-interface event-interface
@@ -436,6 +445,9 @@
 	  channel-ready?
 	  channel-read-block
 	  channel-write-block
+	  channel-buffer-size
+	  channel-crlf?
+	  channel-console-encoding
 	  channel-abort
 	  ))
 
@@ -448,12 +460,16 @@
 
 	  ; called on startup
 	  install-shared-bindings!+gc
-	  
-	  ; for writing images
-	  s48-cleaned-imported-bindings
-	  s48-exported-bindings
 	  ))
 
+(define external-events-interface
+  (export initialize-external-events
+	  s48-permanent-external-event-uid
+	  s48-external-event-uid s48-unregister-external-event-uid
+	  s48-external-event-ready?/unsafe s48-external-event-pending?/unsafe
+	  s48-note-external-event!/unsafe
+	  s48-dequeue-external-event!/unsafe))
+
 (define-interface bignum-low-interface
   (export initialize-bignums
 
@@ -491,7 +507,7 @@
 	  bignum-bitwise-and
 	  bignum-bitwise-ior
 	  bignum-bitwise-xor
-	  long->bignum
+	  long->bignum unsigned-long->bignum
 	  bignum-bits-to-size))
 
 (define-interface integer-arithmetic-interface
@@ -514,6 +530,7 @@
 	  integer-bitwise-ior
 	  integer-bitwise-xor
 	  enter-integer
+	  enter-unsigned-integer
 	  long-as-integer-size))
 
 (define-interface interpreter-internal-interface
@@ -523,7 +540,7 @@
 	  raise
 	  push-exception-setup!
 
-	  any-> string-> boolean-> fixnum-> vm-integer-> char->
+	  any-> string-> boolean-> fixnum-> vm-integer-> char-> char-scalar-value->
 	  vector-> code-vector-> 
 	  input-type no-coercion
 	  
@@ -593,6 +610,11 @@
 	  s48-add-os-signal
           s48-os-signal-pending
 	  s48-reset-interrupts!
+
+	  s48-external-event-uid s48-permanent-external-event-uid s48-unregister-external-event-uid
+	  s48-external-event-pending?/unsafe s48-external-event-ready?/unsafe
+	  s48-note-external-event!/unsafe
+	  s48-dequeue-external-event!/unsafe
 	  
           s48-define-exported-binding
 	  s48-get-imported-binding
@@ -621,6 +643,7 @@
 	  s48-allocate-bignum
  	  s48-shorten-bignum
  	  s48-enter-integer
+	  s48-enter-unsigned-integer
 
 	  ; for initializing additional processes
 	  s48-reset-external-roots!
@@ -646,9 +669,185 @@
 	  s48-integer-or-floanum->
 	  s48-integer-or-floanum-<=
 	  s48-integer-or-floanum->=
+          s48-integer-remainder
+          s48-integer-quotient
 	  s48-integer-bitwise-not
 	  s48-integer-bit-count
 	  s48-integer-bitwise-and
 	  s48-integer-bitwise-ior
 	  s48-integer-bitwise-xor
+
+	  s48-string-set
+	  s48-string-ref
+	  s48-string-length
+	  s48-allocate-string
+	  s48-enter-string-latin-1
+	  s48-enter-string-latin-1-n
+	  s48-copy-latin-1-to-string-n
+	  s48-copy-latin-1-to-string
+	  s48-copy-string-to-latin-1
+	  s48-copy-string-to-latin-1-n
+	  s48-string-utf-8-length
+	  s48-string-utf-8-length-n
+	  s48-enter-string-utf-8
+	  s48-enter-string-utf-8-n
+	  s48-copy-string-to-utf-8
+	  s48-copy-string-to-utf-8-n
+	  ))
+
+; Reading and writing images
+
+(define-interface read-image-interface
+  (export s48-read-image
+
+	  s48-startup-procedure
+	  s48-initial-symbols
+	  s48-initial-imported-bindings
+	  s48-initial-exported-bindings
+	  s48-resumer-records
+
+	  s48-initialization-complete!
+	  s48-initializing-gc-root
+
+	  s48-set-image-values!
+	  ))
+
+(define-interface write-image-interface
+  (export s48-write-image))
+
+(define-interface write-image-gc-specific-interface
+  (export deallocate-areas
+	  write-header
+	  begin-making-image/gc-specific
+	  allocate-new-image-object
+	  finalize-new-image-object
+	  write-image-areas
+	  image-alloc
+	  adjust-descriptors!
+	  note-traced-last-stob!))
+
+(define-interface read-image-gc-specific-interface
+  (export really-read-image
+	  initialize-image-areas!))
+
+(define-interface read-image-portable-interface
+  (export really-read-image-portable))
+
+; This is used by WRITE-IMAGE and READ-IMAGE-PORTABLE
+(define-interface image-table-interface
+  (export make-image-location
+          image-location-new-descriptor
+	  set-image-location-new-descriptor!
+          image-location-next
+          set-image-location-next!
+
+          make-table
+	  deallocate-table
+	  break-table!
+	  table-okay?
+          table-set!
+          table-ref
+	  
+	  table-find   ;; debugging
+	  table-keys
+	  table-values
+	  table-count
+	  table-size
+))
+
+(define-interface image-util-interface
+  (export (area-type-size :syntax)
+	  (image-format :syntax)
+	  valid-image-format?))
+
+(define-interface read-image-util-interface
+  (export init-read-image!
+	  set-status!
+	  get-status
+	  is-eof?
+	  (read-check :syntax)
+	  got-error?
+	  image-read-block
+	  read-lost
+	  read-page
+	  read-newline
+	  read-this-character
+	  reverse-descriptor-byte-order!
+	  reverse-byte-order!
+	  adjust
+	  relocate-image
+	  alloc-space
+	  relocate-symbol-table-two-space!
+	  relocate-binding-table-two-space!
+
+	  set-startup-procedure!
+	  get-startup-procedure
+	  set-symbols!
+	  get-symbols
+	  set-imported-bindings!
+	  get-imported-bindings
+	  set-exported-bindings!
+	  get-exported-bindings
+	  set-resumer-records!
+	  get-resumer-records
+
+	  set-img-start-addr!
+	  get-img-start-addr
+	  set-img-end-addr!
+	  get-img-end-addr
+	  set-img-heap-size!
+	  get-img-heap-size
+
+	  set-small-img-start-addr!
+	  get-small-img-start-addr
+	  set-small-img-hp-addr!
+	  get-small-img-hp-addr
+	  set-small-img-end-addr!
+	  get-small-img-end-addr
+	  set-small-img-heap-size!
+	  get-small-img-heap-size
+
+	  set-large-img-start-addr!
+	  get-large-img-start-addr
+	  set-large-img-hp-addr!
+	  get-large-img-hp-addr
+	  set-large-img-end-addr!
+	  get-large-img-end-addr
+	  set-large-img-heap-size!
+	  get-large-img-heap-size
+
+	  set-weaks-img-start-addr!
+	  get-weaks-img-start-addr
+	  set-weaks-img-hp-addr!
+	  get-weaks-img-hp-addr
+	  set-weaks-img-end-addr!
+	  get-weaks-img-end-addr
+	  set-weaks-img-heap-size!
+	  get-weaks-img-heap-size
+	  ))
+
+(define-interface read-image-util-gc-specific-interface
+  (export get-small-start-addr
+	  get-large-start-addr
+	  get-weaks-start-addr))
+
+(define-interface write-image-util-interface
+  (export write-page
+	  (write-check :syntax)
+	  write-header-integer
+	  image-write-init
+	  image-write-terminate
+	  image-write-status
+	  write-descriptor
+	  write-image-block
+	  empty-image-buffer!))
+
+; for debugging
+; To activate/deactivate it, the flag 'debug-mode?' must be set in
+; debugging.scm
+(define-interface debugging-interface
+  (export debug      ;; strings
+          debug-int  ;; integers
+          debug-line ;; "--------"
+	  debug-mode?
 	  ))
