@@ -1,4 +1,4 @@
-; Copyright (c) 1993-2005 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
 
 ; Code to handle the calling and return protocols.
 
@@ -28,7 +28,7 @@
 	       (goto run-body-with-default-space code 2 template))
 	      ((= (native->byte-protocol protocol)
 		  stack-arg-count)
-	       (goto call-native-code *val* 2 stack-arg-count))
+	       (goto call-native-code-with-default-space 2))
 	      (else
 	       (goto plain-protocol-match stack-arg-count))))
       (goto application-exception
@@ -47,8 +47,20 @@
 (define (byte->native-protocol protocol)
   (bitwise-ior protocol 128))
 
-(define (call-native-code proc protocol-skip stack-nargs)
-  (goto post-native-dispatch (s48-call-native-procedure proc protocol-skip)))
+(define (call-native-code-with-default-space protocol-skip)
+  (if (and (ensure-default-procedure-space!)
+	   (pending-interrupt?))
+      (goto handle-native-interrupt protocol-skip)
+      (goto really-call-native-code protocol-skip)))
+
+(define (call-native-code protocol-skip needed-stack-space)
+  (if (and (ensure-stack-space! needed-stack-space)
+	   (pending-interrupt?))
+      (goto handle-native-interrupt protocol-skip)
+      (goto really-call-native-code protocol-skip)))      
+
+(define (really-call-native-code protocol-skip)
+  (goto post-native-dispatch (s48-call-native-procedure *val* protocol-skip)))
 
 (define s48-*native-protocol*)
 
@@ -59,6 +71,8 @@
   (set! s48-*native-protocol* protocol))
 
 (define (post-native-dispatch tag)
+;  (write-string "P" (current-error-port))
+;  (write-integer tag (current-error-port))
   (let loop ((tag tag))
     (case tag
       ((0)
@@ -66,23 +80,22 @@
       ((1)
        (goto perform-application s48-*native-protocol*))
       ((2)
-       (cond ((pending-interrupt?)
-	      (push-native-interrupt-continuation)
-	      (goto find-and-call-interrupt-handler))
-	     (else
-	      (loop (s48-invoke-native-continuation
-		     (+ (address->integer (pop-continuation-from-stack))
-			2))))))
+       (let* ((template (pop))
+              (return-address (pop)))
+         (cond ((pending-interrupt?)
+                ;(write-string "interrupt is pending" (current-error-port))
+                (goto handle-native-poll template return-address))
+               (else
+                ;(write-string "no interrupt is pending" (current-error-port))
+                (loop (s48-jump-native return-address template))))))
       ((3)
-       ;;; just a prototype
-       (push-native-interrupt-continuation)
-       (let ((nargs (pop)))
-	 (write-string "exception with " (current-error-port))
-	 (write-integer nargs (current-error-port))
-	 (write-string " arguments" (current-error-port))
-	 (goto raise nargs)))
+       (goto return-values s48-*native-protocol* null 0))
       ((4)
        (goto interpret *code-pointer*))
+      ((5)
+       (goto do-apply-with-native-cont s48-*native-protocol* (pop)))
+      ((6)
+       (raise-exception trap 0 *val*))
       (else
        (error "unexpected native return value" tag)))))
 
@@ -114,6 +127,12 @@
 					      return-pointer-offset)
 				    stack-arg-count))))
 
+(define (maybe-make-native-continuation stack-arg-count maybe-cont)
+  (if (= maybe-cont 0)
+      (move-args-above-cont! stack-arg-count)
+      (make-continuation-on-stack (integer->address maybe-cont)
+                                  stack-arg-count)))
+
 ; Call a template instead of a procedure.  This is currently used for
 ; stringing together code for top-level forms and doing the same thing for
 ; the initialization code made by the static linker.
@@ -164,6 +183,21 @@
 	  (let ((args (pop-args->list*+gc list-args stack-nargs)))
 	    (raise-exception wrong-type-argument -1 *val* args))))))
 
+(define (do-apply-with-native-cont stack-nargs maybe-cont)
+  (let ((list-args (pop)))
+    (receive (okay? length)
+	(okay-argument-list list-args)
+      (if okay?
+	  (begin
+	    (maybe-make-native-continuation stack-nargs maybe-cont)
+	    (goto perform-application-with-rest-list
+		  stack-nargs
+		  list-args
+		  length))
+	  (let ((args (pop-args->list*+gc list-args stack-nargs)))
+	    (raise-exception wrong-type-argument -1 *val* args))))))
+  
+
 ; This is only used for the closed-compiled version of APPLY.
 ;
 ; Stack = arg0 arg1 ... argN rest-list N+1 total-arg-count
@@ -343,7 +377,7 @@
   	       (native? #f))
       (let ((win (lambda (skip stack-arg-count)
   		   (if native?
-  		       (goto call-native-code *val* skip stack-arg-count)
+  		       (goto call-native-code skip stack-space)
 		       (let ((template (closure-template *val*)))
 		         (goto run-body (template-code template)
 			       skip
@@ -468,13 +502,28 @@
       (goto handle-interrupt)
       (goto interpret *code-pointer*)))
 
-(define (env-and-template-setup env/template template)
-  (cond ((= #b11 env/template)
+(define (env-and-template-setup spec template)
+  (cond ((= #b011 spec)
 	 (push (closure-env *val*))
 	 (push template))
-	((= #b01 env/template)
+	((= #b001 spec)
+	 (push template))
+	((= #b010 spec)
 	 (push (closure-env *val*)))
-	((= #b10 env/template)
+	;; the next two are for the output of the optimizer,
+	;; for closures that have the environment merged in
+	((= #b100 spec)
+	 (push *val*))  ; closure
+	((= #b110 spec)
+	 (push *val*)
+	 (push (closure-env *val*)))
+	;; the following probably won't occur in the wild
+	((= #b101 spec)
+	 (push *val*)
+	 (push template))
+	((= #b111 spec)
+	 (push *val*)
+	 (push (closure-env *val*))
 	 (push template))))
 
 ;----------------------------------------------------------------
@@ -677,8 +726,9 @@
 (define (native-return protocol-skip)
   (goto post-native-dispatch
 	(s48-invoke-native-continuation
-	  (+ (address->integer (pop-continuation-from-stack))
-	     protocol-skip))))
+         (address->integer (pop-continuation-from-stack))
+         protocol-skip)))
+        
 
 ; The continuation wants a fixed number of arguments.  We pop the current
 ; continuation, move the stack arguments down to the new stack top, push
