@@ -1,5 +1,5 @@
 ; -*- Mode: Scheme; Syntax: Scheme; Package: Scheme; -*-
-; Copyright (c) 1993-2005 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
 
 ; The stack grows from higher addresses to lower ones.
 ; *STACK-BEGIN* and *STACK-END* delimit the stack portion of memory.
@@ -103,6 +103,7 @@
 	(store! a stack-marker))
       (let* ((key (ensure-space return-code-size))
 	     (code (make-return-code bottom-of-stack-protocol
+                                     #xffff     ; dummy template offset
 				     0		; opcode (never executed)
 				     0		; size
 				     key)))
@@ -332,6 +333,9 @@
   (set! *cont* (stack-cont-continuation *cont*))
   (integer->address (pop)))
 
+(define (set-cont-to-stack!)
+  (set! *cont* *stack*))
+
 ; Pushing exception data.  We do this in a chunk to ensure that we agree
 ; with the VM as to which value is where.
 
@@ -459,12 +463,11 @@
 	       (write-integer (extract-fixnum name) out))
 	      ((and (record? name)
 		    (vm-string? (record-ref name 2)))
-	       (write-string (extract-string (record-ref name 2)) out))
+	       (write-vm-string (record-ref name 2) out))
 	      ((and (record? name)
 		    (vm-symbol? (record-ref name 2)))
-	       (write-string (extract-string
-			      (vm-symbol->string (record-ref name 2)))
-			     out))
+	       (write-vm-string (vm-symbol->string (record-ref name 2))
+				out))
 	      (else
 	       (write-string "?" out))))
       (write-string " ?? " out))
